@@ -11,6 +11,7 @@
     #include <QtGui/QX11Info>
     #include <X11/Xlib.h>
     #include <X11/Xatom.h>
+    #include <QMaemo5InformationBox>
 #endif
 
 #include <QDebug>
@@ -23,18 +24,14 @@ MainWindow::MainWindow(QWidget *parent) :
 {
     m_pUI->setupUi(this);
 
-      // the zoom values are chosen to be like in Mozilla Firefox 3
+    /*
     m_zoomLevels << 30 << 50 << 67 << 80 << 90;
     m_zoomLevels << 100;
     m_zoomLevels << 110 << 120 << 133 << 150 << 170 << 200 << 240 << 300;
-
     m_currentZoom = 100;
+    */
 
-    connect(m_pUI->webView, SIGNAL(loadStarted()), this, SLOT(showLoading()));
-    connect(m_pUI->webView, SIGNAL(urlChanged(QUrl)), this, SLOT(changeUrl(QUrl)));
-    connect(m_pUI->webView, SIGNAL(loadFinished(bool)), this, SLOT(hideLoading(bool)));
-    connect(m_pUI->webView->page()->networkAccessManager(), SIGNAL(finished(QNetworkReply*)),
-        this, SLOT(finishedRequest(QNetworkReply*)));
+    m_zoomFactor = 1;
 
     // loading label...
     m_pLoadingLabel = new QLabel(this);
@@ -79,8 +76,32 @@ MainWindow::MainWindow(QWidget *parent) :
 
     loadSettings();
 
+
+    // set web page
     m_webPage.mainFrame()->setScrollBarPolicy(Qt::Vertical, Qt::ScrollBarAlwaysOff);
     m_webPage.mainFrame()->setScrollBarPolicy(Qt::Horizontal, Qt::ScrollBarAlwaysOff);
+
+    connect(m_webPage.networkAccessManager(), SIGNAL(finished(QNetworkReply*)),
+        this, SLOT(finishedRequest(QNetworkReply*)));
+
+    // m_webPage.setLinkDelegationPolicy(QWebPage::DelegateAllLinks);
+    // m_webPage.settings()->setAttribute(QWebSettings::PluginsEnabled, true);
+    // m_webPage.setForwardUnsupportedContent(true);
+    m_webPage.settings()->setAttribute(QWebSettings::DeveloperExtrasEnabled, true);
+
+    // offline?
+    // QWebSettings::OfflineStorageDatabaseEnabled
+    // QWebSettings::OfflineWebApplicationCacheEnabled
+    // QWebSettings::enablePersistentStorage(path);
+
+    // connect(&m_webPage, SIGNAL(linkClicked(QUrl)), this, SLOT(linkClicked(QUrl)));
+    // connect(&m_webPage, SIGNAL(unsupportedContent(QNetworkReply*)), this, SLOT(unsupportedContent(QNetworkReply*)));
+
+    // set webview
+    connect(m_pUI->webView, SIGNAL(loadStarted()), this, SLOT(showLoading()));
+    connect(m_pUI->webView, SIGNAL(urlChanged(QUrl)), this, SLOT(changeUrl(QUrl)));
+    connect(m_pUI->webView, SIGNAL(loadFinished(bool)), this, SLOT(hideLoading(bool)));
+
     m_pUI->webView->setPage(&m_webPage);
 
     m_pUI->webView->setWindowTitle("Macuco");
@@ -156,6 +177,11 @@ void MainWindow::showLoading() {
 
 void MainWindow::changeUrl(QUrl url)
 {
+    /*
+    QIcon icon = m_webPage.settings()->iconForUrl(url);
+    qDebug() << icon.isNull();
+    */
+
     m_currentUrl = url;
 
 #ifndef Q_WS_MAEMO_5
@@ -178,6 +204,14 @@ void MainWindow::hideLoading(bool success) {
     // ...
 
     if ( !success ) {
+        QString msg("<b>Loading Error</b><br/>");
+        if ( m_errors.length()>1 ) msg+=m_errors.join("<br/>"); else msg+=m_currentUrl.toString();
+        QMaemo5InformationBox::information(this,
+            msg,
+            QMaemo5InformationBox::DefaultTimeout
+        );
+
+        /*
         if ( QMessageBox::warning(
             this,
             "Loading Error",
@@ -188,6 +222,8 @@ void MainWindow::hideLoading(bool success) {
             // retry
             reload();
         }
+        */
+        return;
     }
 
 #else
@@ -221,14 +257,15 @@ void MainWindow::adjustSizes() {
     m_pMiniToolbar->setGeometry( 0, height() - 38, 30, 38 );
 #endif
 
-    if ( m_webPage.isIphoneUserAgent() ) {
-        disableWebkitAutoSize();
+    if ( m_webPage.isIphoneUserAgent() ) zoomFit();
 
-        if ( width() > height() ) {
-            m_pUI->webView->setZoomFactor(qreal(width())/480.0);
-        } else {
-            m_pUI->webView->setZoomFactor(qreal(height())/480.0);
-        }
+}
+
+void MainWindow::zoomFit() {
+    if ( width() > height() ) {
+        setZoom(qreal(width())/480.0);
+    } else {
+        setZoom(qreal(height())/480.0);
     }
 }
 
@@ -254,30 +291,24 @@ void MainWindow::finishedRequest(QNetworkReply* reply)
 {
     if (reply->error() == QNetworkReply::NoError) return;
 
-    qDebug() << reply->errorString();
     m_errors << reply->errorString();
+    qDebug() << "Error(s): " << m_errors;
 }
 
-void MainWindow::zoomIn()
-{
-    disableWebkitAutoSize();
-
-    int i = m_zoomLevels.indexOf(m_currentZoom);
-    if (i < m_zoomLevels.count() - 1)
-        m_currentZoom = m_zoomLevels[i + 1];
-
-    m_pUI->webView->setZoomFactor(qreal(m_currentZoom)/100.0);
+void MainWindow::zoomIn() {
+    setZoom(m_zoomFactor*1.15);
 }
 
-void MainWindow::zoomOut()
-{
+void MainWindow::zoomOut() {
+    setZoom(m_zoomFactor/1.15);
+}
+
+void MainWindow::setZoom(qreal zoomFactor) {
     disableWebkitAutoSize();
+    m_zoomFactor = zoomFactor;
+    if ( m_zoomFactor>.90 && m_zoomFactor<1.1) m_zoomFactor =1;
 
-    int i = m_zoomLevels.indexOf(m_currentZoom);
-    if (i > 0)
-        m_currentZoom = m_zoomLevels[i - 1];
-
-    m_pUI->webView->setZoomFactor(qreal(m_currentZoom)/100.0);
+    m_pUI->webView->setZoomFactor(m_zoomFactor);
 }
 
 void MainWindow::keyPressEvent(QKeyEvent *event)
@@ -310,6 +341,16 @@ void MainWindow::keyPressEvent(QKeyEvent *event)
             if ( (event->modifiers()&Qt::ControlModifier) != Qt::ControlModifier  ) break;
             setFullScreen(true);
             break;
+
+        case Qt::Key_O:
+            if ( (event->modifiers()&Qt::ControlModifier) != Qt::ControlModifier  ) break;
+            setZoom(1);
+            break;
+
+        case Qt::Key_I:
+            if ( (event->modifiers()&Qt::ControlModifier) != Qt::ControlModifier  ) break;
+            zoomFit();
+            break;
     }
 }
 
@@ -514,10 +555,18 @@ void MainWindow::hideToolbar()
 void MainWindow::on_actionUser_Agent_triggered()
 {
     setUserAgent(!m_webPage.isIphoneUserAgent());
-    reload();
+    // reload();
 }
 
 void MainWindow::setUserAgent(bool iphone/*=true*/) {
     m_webPage.setIPhoneUserAgent(iphone);
     m_pUI->actionUser_Agent->setText(iphone ? "Default User Agent" : "iPhone User Agent");
 }
+
+void MainWindow::linkClicked( const QUrl & url ) {
+    qDebug() << "Link clicked: " << url.toString();
+}
+
+void MainWindow::unsupportedContent(QNetworkReply* pReply) {
+    qDebug() << "unsupportedContent: " << pReply;
+}
